@@ -19,10 +19,10 @@ enum bt_asha_state_t {
 	ASHA_STOPPED = 0,
 	ASHA_STARTING,
 	ASHA_STARTED,
-	ASHA_STOPPING,
 };
 
 typedef void (*bt_asha_cb_t)(int status, void *data);
+typedef void (*bt_asha_attach_cb_t)(void *data);
 
 struct bt_asha {
 	struct bt_gatt_client *client;
@@ -43,8 +43,11 @@ struct bt_asha {
 	int8_t volume;
 
 	enum bt_asha_state_t state;
-	bt_asha_cb_t cb;
-	void *cb_user_data;
+	bt_asha_cb_t state_cb;
+	void *state_cb_data;
+
+	bt_asha_attach_cb_t attach_cb;
+	void *attach_cb_data;
 };
 
 struct bt_asha_set {
@@ -60,10 +63,10 @@ void bt_asha_free(struct bt_asha *asha);
 
 unsigned int bt_asha_start(struct bt_asha *asha, bt_asha_cb_t cb,
 							void *user_data);
-unsigned int bt_asha_stop(struct bt_asha *asha, bt_asha_cb_t cb,
-							void *user_data);
+unsigned int bt_asha_stop(struct bt_asha *asha);
 
 bool bt_asha_set_volume(struct bt_asha *asha, int8_t volume);
 
-bool bt_asha_probe(struct bt_asha *asha, struct gatt_db *db,
-						struct bt_gatt_client *client);
+bool bt_asha_attach(struct bt_asha *asha, struct gatt_db *db,
+		struct bt_gatt_client *client, bt_asha_attach_cb_t probe_cb,
+							void *cb_user_data);
