@@ -22,7 +22,7 @@
 #include <fcntl.h>
 #include <poll.h>
 
-#include "lib/bluetooth.h"
+#include "bluetooth/bluetooth.h"
 
 #include "src/shared/ad.h"
 #include "src/shared/btp.h"
@@ -328,7 +328,7 @@ static void null_evt(const void *data, uint16_t size)
 }
 
 static const struct indexstr_data error_table[] = {
-	{ 0x01, "Faile" },
+	{ 0x01, "Failed" },
 	{ 0x02, "Unknown Command" },
 	{ 0x03, "Not Ready" },
 	{ 0x04, "Invalid Index" },
