@@ -24,7 +24,7 @@
 #include "parser.h"
 #include "sdp.h"
 #include "l2cap.h"
-#include "lib/hci.h"
+#include "bluetooth/hci.h"
 
 typedef struct {
 	uint16_t handle;
@@ -279,7 +279,7 @@ static char *status2str(uint16_t status)
 {
 	switch (status) {
 	case 0x0000:
-		return "No futher information available";
+		return "No further information available";
 	case 0x0001:
 		return "Authentication pending";
 	case 0x0002:
