@@ -1,27 +1,8 @@
+// SPDX-License-Identifier: BSD-2-Clause
 /*
  * Copyright (c) 2013, Kenneth MacKay
  * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are
- * met:
- *  * Redistributions of source code must retain the above copyright
- *   notice, this list of conditions and the following disclaimer.
- *  * Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,33 +18,33 @@
 #include <fcntl.h>
 
 #include "src/shared/ecc.h"
+#include "src/shared/util.h"
+#include "src/shared/tester.h"
 
-static void vli_print(uint8_t *vli, size_t size)
+static void print_dump(const char *str, void *user_data)
 {
-	while (size) {
-		printf("%02X ", vli[size - 1]);
-		size--;
-	}
+	tester_debug("%s", str);
+}
+
+static void print_buf(const char *label, uint8_t *buf, size_t len)
+{
+	tester_debug("%s", label);
+
+	util_hexdump(' ', buf, len, print_dump, NULL);
 }
 
 #define PAIR_COUNT 200
 
-static void test_multi(void)
+static void test_multi(const void *data)
 {
 	uint8_t public1[64], public2[64];
 	uint8_t private1[32], private2[32];
 	uint8_t shared1[32], shared2[32];
 	int i;
 
-	if (g_test_verbose())
-		g_print("Testing %u random private key pairs", PAIR_COUNT);
+	tester_debug("Testing %u random private key pairs", PAIR_COUNT);
 
 	for (i = 0; i < PAIR_COUNT; i++) {
-		if (g_test_verbose()) {
-			printf(".");
-			fflush(stdout);
-		}
-
 		ecc_make_key(public1, private1);
 		ecc_make_key(public2, private2);
 
@@ -71,37 +52,20 @@ static void test_multi(void)
 		ecdh_shared_secret(public2, private1, shared2);
 
 		if (memcmp(shared1, shared2, sizeof(shared1)) != 0) {
-			printf("Shared secrets are not identical!\n");
-			printf("Shared secret 1 = ");
-			vli_print(shared1, sizeof(shared1));
-			printf("\n");
-			printf("Shared secret 2 = ");
-			vli_print(shared2, sizeof(shared2));
-			printf("\n");
-			printf("Private key 1 = ");
-			vli_print(private1, sizeof(private1));
-			printf("\n");
-			printf("Private key 2 = ");
-			vli_print(private2, sizeof(private2));
-			printf("\n");
+			tester_debug("Shared secrets are not identical!\n");
+			print_buf("Shared secret 1 = ", shared1,
+							sizeof(shared1));
+			print_buf("Shared secret 2 = ", shared2,
+							sizeof(shared2));
+			print_buf("Private key 1 = ", private1,
+							sizeof(private1));
+			print_buf("Private key 2 = ", private2,
+							sizeof(private2));
 			g_assert_not_reached();
 		}
 	}
 
-	if (g_test_verbose())
-		printf("\n");
-}
-
-static void print_buf(const char *label, uint8_t *buf, size_t len)
-{
-	size_t i;
-
-	printf("%s ", label);
-
-	for (i = 0; i < len; i++)
-		printf("%02x", buf[i]);
-
-	printf("\n");
+	tester_test_passed();
 }
 
 static int test_sample(uint8_t priv_a[32], uint8_t priv_b[32],
@@ -111,7 +75,10 @@ static int test_sample(uint8_t priv_a[32], uint8_t priv_b[32],
 	uint8_t dhkey_a[32], dhkey_b[32];
 	int fails = 0;
 
+	memset(dhkey_a, 0, sizeof(dhkey_a));
 	ecdh_shared_secret(pub_b, priv_a, dhkey_a);
+
+	memset(dhkey_b, 0, sizeof(dhkey_b));
 	ecdh_shared_secret(pub_a, priv_b, dhkey_b);
 
 	if (g_test_verbose()) {
@@ -121,27 +88,24 @@ static int test_sample(uint8_t priv_a[32], uint8_t priv_b[32],
 	}
 
 	if (memcmp(dhkey_a, dhkey, 32)) {
-		if (g_test_verbose())
-			printf("DHKey A doesn't match!\n");
+		tester_debug("DHKey A doesn't match!");
 		fails++;
 	} else {
-		if (g_test_verbose())
-			printf("DHKey A matches :)\n");
+		tester_debug("DHKey A matches :)");
 	}
 
+
 	if (memcmp(dhkey_b, dhkey, 32)) {
-		if (g_test_verbose())
-			printf("DHKey B doesn't match!\n");
+		tester_debug("DHKey B doesn't match!");
 		fails++;
 	} else {
-		if (g_test_verbose())
-			printf("DHKey B matches :)\n");
+		tester_debug("DHKey B matches :)");
 	}
 
 	return fails;
 }
 
-static void test_sample_1(void)
+static void test_sample_1(const void *data)
 {
 	uint8_t priv_a[32] = {	0xbd, 0x1a, 0x3c, 0xcd, 0xa6, 0xb8, 0x99, 0x58,
 				0x99, 0xb7, 0x40, 0xeb, 0x7b, 0x60, 0xff, 0x4a,
@@ -183,9 +147,11 @@ static void test_sample_1(void)
 	fails = test_sample(priv_a, priv_b, pub_a, pub_b, dhkey);
 
 	g_assert(fails == 0);
+
+	tester_test_passed();
 }
 
-static void test_sample_2(void)
+static void test_sample_2(const void *data)
 {
 	uint8_t priv_a[32] = {	0x63, 0x76, 0x45, 0xd0, 0xf7, 0x73, 0xac, 0xb7,
 				0xff, 0xdd, 0x03, 0x72, 0xb9, 0x72, 0x85, 0xb4,
@@ -227,9 +193,11 @@ static void test_sample_2(void)
 	fails = test_sample(priv_a, priv_b, pub_a, pub_b, dhkey);
 
 	g_assert(fails == 0);
+
+	tester_test_passed();
 }
 
-static void test_sample_3(void)
+static void test_sample_3(const void *data)
 {
 	uint8_t priv_a[32] = {	0xbd, 0x1a, 0x3c, 0xcd, 0xa6, 0xb8, 0x99, 0x58,
 				0x99, 0xb7, 0x40, 0xeb, 0x7b, 0x60, 0xff, 0x4a,
@@ -256,17 +224,54 @@ static void test_sample_3(void)
 	fails = test_sample(priv_a, priv_a, pub_a, pub_a, dhkey);
 
 	g_assert(fails == 0);
+
+	tester_test_passed();
+}
+
+static void test_invalid_pub(const void *data)
+{
+	uint8_t priv_a[32] = {	0xbd, 0x1a, 0x3c, 0xcd, 0xa6, 0xb8, 0x99, 0x58,
+				0x99, 0xb7, 0x40, 0xeb, 0x7b, 0x60, 0xff, 0x4a,
+				0x50, 0x3f, 0x10, 0xd2, 0xe3, 0xb3, 0xc9, 0x74,
+				0x38, 0x5f, 0xc5, 0xa3, 0xd4, 0xf6, 0x49, 0x3f,
+	};
+	uint8_t pub_a[64] = {	0xe6, 0x9d, 0x35, 0x0e, 0x48, 0x01, 0x03, 0xcc,
+				0xdb, 0xfd, 0xf4, 0xac, 0x11, 0x91, 0xf4, 0xef,
+				0xb9, 0xa5, 0xf9, 0xe9, 0xa7, 0x83, 0x2c, 0x5e,
+				0x2c, 0xbe, 0x97, 0xf2, 0xd2, 0x03, 0xb0, 0x20,
+
+				0x8b, 0xd2, 0x89, 0x15, 0xd0, 0x8e, 0x1c, 0x74,
+				0x24, 0x30, 0xed, 0x8f, 0xc2, 0x45, 0x63, 0x76,
+				0x5c, 0x15, 0x52, 0x5a, 0xbf, 0x9a, 0x32, 0x63,
+				0x6d, 0xeb, 0x2a, 0x65, 0x49, 0x9c, 0x80, 0xdc,
+	};
+	uint8_t dhkey[32] = {	0x2d, 0xab, 0x00, 0x48, 0xcb, 0xb3, 0x7b, 0xda,
+				0x55, 0x7b, 0x8b, 0x72, 0xa8, 0x57, 0x87, 0xc3,
+				0x87, 0x27, 0x99, 0x32, 0xfc, 0x79, 0x5f, 0xae,
+				0x7c, 0x1c, 0xf9, 0x49, 0xe6, 0xd7, 0xaa, 0x70,
+	};
+	int fails;
+
+	memset(pub_a + 32, 0x42, 32);
+
+	fails = test_sample(priv_a, priv_a, pub_a, pub_a, dhkey);
+
+	g_assert(fails >= 1);
+
+	tester_test_passed();
 }
 
 int main(int argc, char *argv[])
 {
-	g_test_init(&argc, &argv, NULL);
+	tester_init(&argc, &argv);
 
-	g_test_add_func("/ecdh/multi", test_multi);
+	tester_add("/ecdh/multi", NULL, NULL, test_multi, NULL);
 
-	g_test_add_func("/ecdh/sample/1", test_sample_1);
-	g_test_add_func("/ecdh/sample/2", test_sample_2);
-	g_test_add_func("/ecdh/sample/3", test_sample_3);
+	tester_add("/ecdh/sample/1", NULL, NULL, test_sample_1, NULL);
+	tester_add("/ecdh/sample/2", NULL, NULL, test_sample_2, NULL);
+	tester_add("/ecdh/sample/3", NULL, NULL, test_sample_3, NULL);
 
-	return g_test_run();
+	tester_add("/ecdh/invalid", NULL, NULL, test_invalid_pub, NULL);
+
+	return tester_run();
 }
