@@ -18,8 +18,8 @@
 
 #include <glib.h>
 
-#include "lib/bluetooth.h"
-#include "lib/mgmt.h"
+#include "bluetooth/bluetooth.h"
+#include "bluetooth/mgmt.h"
 
 #include "src/shared/mgmt.h"
 
